/* $Id: SimplePlayer.c,v 1.4 2001/02/21 02:05:38 cvs Exp $
 *
 * This is a simple program which allows us to control the
 * small RC cars. It uses ncurses for char io and the numpad
 */

//#define THREAD
//If the following line is uncommented the client will dump its output to the screen
//rather than to a server

#include <stdio.h>
#include <stdlib.h>
#include <curses.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netdb.h>
#include <string.h>
#include "rcx-com.h"
#include "rcx_client_constants.h"


#define DefaultPort     10022


char **RobotNames;



WINDOW *scr;
int id;
int NumberOfRobotsWeControl;
int sock;			//socket descriptor

void
DieWithError (char *ErrorMessage)
{
  printf ("Fatal Error: %s\n", ErrorMessage);
  exit (1);
}

void
BroadCast (char *SendString, struct sockaddr_in CommandServerAddress)
{
  if (sendto
      (sock, SendString, strlen (SendString), 0,
       (struct sockaddr *) &CommandServerAddress,
       sizeof (CommandServerAddress)) != strlen (SendString))
    DieWithError
      ("BroadCast() sent a different number of bytes than expected");

  free (SendString);

}

//send off a command to the server
char *
sendCommand (RCOMMAND volatile currentCommand)
{
  char *CommandString;
  int SpecialCode;
  CommandString = (char *) malloc (sizeof (char *) * 64);

  memset (CommandString, 0, sizeof (CommandString));

  //  printw("Command for Robot Angle=%d Distance=%d\n",currentCommand.angle,currentCommand.distance);

  //Check if we need to do something special
  if (currentCommand.angle == SPIN_LEFT || currentCommand.angle == SPIN_RIGHT)
    {
      SpecialCode = currentCommand.angle + currentCommand.distance;
      switch (SpecialCode)
	{
	case (SPIN_LEFT + SPIN_LEFT):
	  sprintf (CommandString, "%s,%s %s\n", RobotNames[id], "Kick",
		   "Left");
	  break;
	case (SPIN_RIGHT + SPIN_RIGHT):
	  sprintf (CommandString, "%s,%s %s\n", RobotNames[id], "Kick",
		   "Right");
	  break;
	case (SPIN_RIGHT + SPIN_LEFT):
	  sprintf (CommandString, "%s,%s %s\n", RobotNames[id], "Kick",
		   "Straight");
	  break;
	}
    }
  else
    sprintf (CommandString, "%s,%f,%f\n", RobotNames[id],
	     currentCommand.angle, currentCommand.distance);

  printw ("Sent DataGram Packet: %s", CommandString);

  currentCommand.angle = IGNORE_COMMAND;
  currentCommand.distance = IGNORE_COMMAND;

  return (CommandString);
}




int
main (int argc, char *argv[])
{
  int ch = 0;
  int done = 0;



  char *DEFAULT_ROBOT_NAME;
  static RCOMMAND volatile currentCommand;
  char *TempString;
  struct sockaddr_in broadcastAddr;
  char *broadcastIP;

  unsigned short broadcastPort;
  int broadcastPermission;



  int x = 0;

  broadcastIP = "255.255.255.255";
  DEFAULT_ROBOT_NAME = "SoccerGenius";
  TempString = "";



//Set the defaults
  broadcastPort = DefaultPort;

//Find out how many robots are in our args
  for (x = 1; x < argc; x++)
    {
      if (strcmp (argv[x], "--robot") == 0)
	{
	  NumberOfRobotsWeControl++;
	}
    }

  RobotNames = (char **) malloc (NumberOfRobotsWeControl * sizeof (char));



  {

    int WhichRobot = 0;

    for (x = 1; x < argc; x++)
      {
	if (strcmp (argv[x], "--robot") == 0)
	  {
	    TempString = (char *) malloc (25);
	    strcpy (TempString, argv[x + 1]);
	    RobotNames[WhichRobot] = TempString;

	    printf ("Robot %i has name %s\n", WhichRobot,
		    RobotNames[WhichRobot]);
	    WhichRobot++;
	  }
	if (strcmp (argv[x], "--port") == 0)
	  {
	    broadcastPort = atoi (argv[x + 1]);
	  }

      }

  }

  if (NumberOfRobotsWeControl == 0)
    {
      RobotNames = (char **) malloc (sizeof (char));
      TempString = (char *) malloc (25);
      strcpy (TempString, DEFAULT_ROBOT_NAME);
      RobotNames[0] = TempString;
    }


  printf ("About to Call Socket\n");

  if ((sock = socket (PF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0)
    {
      DieWithError ("socket() failed");
    }
  else
    {
      printf ("Socket() Call was Successful\n");
    }



//Construct server address structure
  broadcastPermission = 1;
  if (setsockopt
      (sock, SOL_SOCKET, SO_BROADCAST, (void *) &broadcastPermission,
       sizeof (broadcastPermission)) < 0)
    {
      DieWithError ("setsockopt failed");
    }

  memset (&broadcastAddr, 0, sizeof (broadcastAddr));
  broadcastAddr.sin_family = AF_INET;
  broadcastAddr.sin_addr.s_addr = inet_addr (broadcastIP);
  broadcastAddr.sin_port = htons (broadcastPort);

  printf ("Broadcast socket configured\n");


//init id to be zero
  id = 0;

//Initialize the screen and get a pointer to it
  if ((scr = initscr ()) == NULL)
    {
      DieWithError ("initscr() failed");
    }



  if (scrollok (stdscr, TRUE) != OK)
    {
      perror ("scrollok() failed\n");
    }

  if (cbreak () != OK)
    {
      perror ("error switching to cbreak mode\n");
    }
  if (noecho () != OK)
    {
      perror ("error switching to noecho mode\n");
    }
  if (keypad (stdscr, TRUE) != OK)
    {
      perror ("error switching to keypad mode\n");
    }



//Reset the command
  currentCommand.distance = IGNORE_COMMAND;
  currentCommand.angle = IGNORE_COMMAND;



//Loop forever reading characters and sending them off
  while (!done)
    {

      //Read a character from the console


      ch = getch ();


      //Set currentCommand parameters according to what key was pressed
      switch (ch)
	{

	case 'i':
	case 'I':
	  id++;
	  if (id >= NumberOfRobotsWeControl)
	    id = id % NumberOfRobotsWeControl;
	  printw ("Now Controlling robot: %s\n", RobotNames[id]);
	  break;
	case 'w':
	case 'W':
	case '8':
	case KEY_UP:
	  currentCommand.distance = FAST;
	  currentCommand.angle = STRAIGHT;
	  break;

	case '2':
	case KEY_DOWN:
	  currentCommand.distance = -FAST;
	  currentCommand.angle = STRAIGHT;
	  break;
	case '9':
	case KEY_A3:
	case 339:
	  currentCommand.distance = SLOW;
	  currentCommand.angle = EASY_RIGHT;
	  break;
	case '7':
	case KEY_A1:
	case 262:
	  currentCommand.distance = SLOW;
	  currentCommand.angle = EASY_LEFT;
	  break;
	case '3':
	case KEY_C3:
	case 338:
	  currentCommand.distance = -SLOW;
	  currentCommand.angle = EASY_LEFT;
	  break;
	case '1':
	case KEY_C1:
	case 360:
	  currentCommand.distance = -SLOW;
	  currentCommand.angle = EASY_RIGHT;
	  break;
	case ' ':
	case KEY_B2:
	case '5':
	  currentCommand.distance = STOP;
	  currentCommand.angle = STRAIGHT;
	  break;
	case 'S':
	case 's':
	  currentCommand.distance = STOP;
	  currentCommand.angle = STOP;
	  break;

	case 'd':
	case 'D':
	case '6':
	case KEY_RIGHT:

	  currentCommand.distance = STOP;
	  currentCommand.angle = SHARP_RIGHT;
	  break;
	case 'a':
	case 'A':
	case '4':
	case KEY_LEFT:
	  currentCommand.distance = STOP;
	  currentCommand.angle = SHARP_LEFT;
	  break;

	case 122:
	  currentCommand.distance = SPIN_LEFT;
	  currentCommand.angle = SPIN_LEFT;
	  break;

	case 120:
	  currentCommand.distance = SPIN_LEFT;
	  currentCommand.angle = SPIN_RIGHT;
	  break;

	case 99:
	  currentCommand.distance = SPIN_RIGHT;
	  currentCommand.angle = SPIN_RIGHT;
	  break;
	case 'Q':
	case 'q':
	case '0':
	  done = 1;
	  break;
	case ERR:
	  break;
	default:
	  printw ("Unknown command character\n");
	}


//Send the command to the server
      BroadCast (sendCommand (currentCommand), broadcastAddr);

//Publish offscreen display to console
      touchwin (scr);
      refresh ();

//      showStatus(id, currentCommands);
    }				//end of the big while loop

  free (TempString);
  free (RobotNames);


  if (endwin () != OK)
    {
      perror ("error switching to noecho mode\n");
    }
  return 0;
}

/* Local Variables: */
/* compile-command: "gcc -Wall -O -o simplePlayer simplePlayer.c -lncurses -lpthread" */
/* End: */
